# Generated by Django 2.2.7 on 2019-11-12 19:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('marketplace', '0003_componentusage_recurring'),
    ]

    operations = [
        migrations.AddField(
            model_name='offeringcomponent',
            name='content_type',
            field=models.ForeignKey(null=True, blank=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.ContentType'),
        ),
        migrations.AddField(
            model_name='offeringcomponent',
            name='object_id',
            field=models.PositiveIntegerField(null=True, blank=True),
        ),
    ]
