<html>
<head lang="en">
    <meta charset="UTF-8">
    <title>A new user account has been created for you.</title>
</head>
<body>
<p>
    Hello!
</p>
<p>
    User with login {{ user.user.username }} and temporary password {{ password }} has been created.<br />
    Please go to <a href="{{ rancher_url }}">management console</a> to change the password.
</p>
</body>
</html>
