# Generated by Django 2.2.6 on 2019-10-23 09:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_jira', '0019_immutable_default_json'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attachment',
            name='file',
            field=models.FileField(upload_to='jira_attachments'),
        ),
    ]
