# Generated by Django 2.2.7 on 2019-12-16 11:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0008_remove_offering_enable_dynamic_components'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='marketplace.Offering'),
        ),
    ]
