body {
    font-size: 16px;
    font-family: 'Source Sans Pro';
    margin: 120px;
}

h1 {
    letter-spacing: 0.5em;
    text-align: center;
    text-transform: uppercase;
}

#logo {
  text-align: center;
  margin-top: 8px;
}

#logo img {
  height: 70px;
}

.text-right {
    text-align: right;
}

.m-t {
    margin-top: 15px;
}

.invoice-table {
    width: 100%;
}

.invoice-table th {
    border-bottom: 3px solid #e7eaec;
    padding: 11px 5px 11px 5px;
    vertical-align: middle;
}

.invoice-table th  {
    text-align: left;
}

.invoice-total {
    width: 100%;
    text-align: right;
}

.invoice-total > tbody > tr > td {
    border: 0 none;
}

.invoice-total > tbody > tr > td:last-child {
    border-bottom: 3px solid #DDDDDD;
    text-align: right;
    width: 15%;
}
