# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-04 10:28
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0006_customer_backend_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='blocked',
            field=models.BooleanField(default=False),
        ),
    ]
