{% extends "admin/base_site.html" %}
{% load i18n l10n admin_urls static %}

{% block extrahead %}
    {{ block.super }}
    {{ media }}
    <script type="text/javascript" src="{% static 'admin/js/cancel.js' %}"></script>
{% endblock %}

{% block bodyclass %}{{ block.super }} app-{{ opts.app_label }} model-{{ opts.model_name }} delete-confirmation delete-selected-confirmation{% endblock %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'admin:app_list' app_label=opts.app_label %}">{{ opts.app_config.verbose_name }}</a>
&rsaquo; <a href="{% url opts|admin_urlname:'changelist' %}">{{ opts.verbose_name_plural|capfirst }}</a>
&rsaquo; {{ description }}
</div>
{% endblock %}

{% block content %}
{% block description %}<h2>{{ confirmation_description }}</h2>{% endblock %}
    <form method="post">{% csrf_token %}
    <div>
    {% for obj in queryset %}
    <input type="hidden" name="_selected_action" value="{{ obj.pk|unlocalize }}">
    {% endfor %}
    <input type="hidden" name="action" value="{{ action_name }}">
    <input type="hidden" name="post" value="yes">
    <input type="hidden" name="confirmed" value="yes">
    <input type="submit" value="{% trans "Yes, I'm sure" %}">
    <a href="#" class="button cancel-link">{% trans "No, take me back" %}</a>
    </div>
    </form>
{% endblock %}
