# Generated by Django 2.2.7 on 2019-12-02 09:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0006_orderitem_output'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='enable_dynamic_components',
            field=models.BooleanField(default=False, help_text='For example, allow to use offering components related to volume types.'),
        ),
    ]
