# Generated by Django 2.2.7 on 2019-12-27 13:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_rancher', '0008_node_annotations'),
    ]

    operations = [
        migrations.AddField(
            model_name='cluster',
            name='runtime_state',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='node',
            name='runtime_state',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]
