{% load staticfiles %}
<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8">
  <title>Waldur MasterMind</title>
  <link rel="icon" type="image/x-icon" href="{% static 'rest_framework_swagger/images/favicon.ico' %}" sizes="16x16" />
  <link href='{% static "rest_framework_swagger/css/typography.css" %}' media='screen' rel='stylesheet' type='text/css'/>
  <link href='{% static "rest_framework_swagger/css/reset.css" %}' media='screen' rel='stylesheet' type='text/css'/>
  <link href='{% static "rest_framework_swagger/css/screen.css" %}' media='screen' rel='stylesheet' type='text/css'/>
  <link href='{% static "rest_framework_swagger/css/reset.css" %}' media='print' rel='stylesheet' type='text/css'/>
  <link href='{% static "rest_framework_swagger/css/print.css" %}' media='print' rel='stylesheet' type='text/css'/>
  {% block extra_styles %}
  {# -- Add any additional CSS scripts here -- #}
  {% endblock %}

  <script src='{% static "rest_framework_swagger/lib/object-assign-pollyfill.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/jquery-1.8.0.min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/jquery.slideto.min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/jquery.wiggle.min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/jquery.ba-bbq.min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/handlebars-2.0.0.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/js-yaml.min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/lodash.min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/backbone-min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/swagger-ui.min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/highlight.9.1.0.pack.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/highlight.9.1.0.pack_extended.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/jsoneditor.min.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/marked.js" %}' type='text/javascript'></script>
  <script src='{% static "rest_framework_swagger/lib/swagger-oauth.js" %}' type='text/javascript'></script>

  <!-- Some basic translations -->
  <!-- <script src='lang/translator.js' type='text/javascript'></script> -->
  <!-- <script src='lang/ru.js' type='text/javascript'></script> -->
  <!-- <script src='lang/en.js' type='text/javascript'></script> -->

</head>

<body class="swagger-section">
{% block header %}
<div id='header'>
  <div class="swagger-ui-wrap">
    <a id="logo" href="/docs"><span class="logo__title">Waldur MasterMind</span></a>
    <form id='api_selector'>
      <input id="input_baseUrl" name="baseUrl" type="hidden"/>
      {% if USE_SESSION_AUTH %}
        {% csrf_token %}
        {% if request.user.is_authenticated %}
          <div class="input">
            {% block user_context_message %}
              {# Override this block to customize #}
              Hello, {{ request.user }}
            {% endblock %}
          </div>
        {% endif %}
        {% block extra_nav %}
          {# Override this block to add more buttons, content to nav bar. #}
        {% endblock %}
      {% endif %}

      {% if USE_SESSION_AUTH %}
        {% if request.user.is_authenticated %}
          <div class='input'><a id="auth" class="header__btn" href="{{ LOGOUT_URL }}?next={{ request.path }}" data-sw-translate>Waldur MasterMind Logout</a></div>
        {% else %}
          <div class='input'><a id="auth" class="header__btn" href="{{ LOGIN_URL }}?next={{ request.path }}" data-sw-translate>Waldur MasterMind Login</a></div>
        {% endif %}
      {% endif %}
      <div id='auth_container'></div>
    </form>
  </div>
</div>
{% endblock %}

<div id="message-bar" class="swagger-ui-wrap" data-sw-translate>&nbsp;</div>
<div id="swagger-ui-container" class="swagger-ui-wrap"></div>

<script id="drs-settings" type="application/json">
{{ drs_settings | safe }}
</script>

<script src='{% static "rest_framework_swagger/init.js" %}' type='text/javascript'></script>
{% block extra_scripts %}
{# -- Add any additional custom JS scripts here -- #}
{% endblock %}
</body>
</html>
