{% extends "admin/base.html" %}
{% load i18n %}
{% load admin_tools_menu_tags %}

{% block title %}{{ title }} | {% trans 'Waldur' %}{% endblock %}

{% block extrastyle %}
{{ block.super }}
{% if user.is_active and user.is_staff or user.is_active and user.is_support %}
{% if not is_popup %}
{% admin_tools_render_menu_css %}
{% endif %}
{% endif %}
{% endblock %}

{% block branding %}
<h1 id="site-name">{% trans 'Waldur administration' %}</h1>
{% endblock %}

{% block nav-global %}
{% if user.is_active and user.is_staff or user.is_active and user.is_support %}
{% if not is_popup %}
{% admin_tools_render_menu %}
{% endif %}
{% endif %}
{% endblock %}
