# Generated by Django 2.2.6 on 2019-10-30 10:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0002_offeringcomponent_use_limit_for_billing'),
    ]

    operations = [
        migrations.AddField(
            model_name='componentusage',
            name='recurring',
            field=models.BooleanField(default=False, help_text='Reported value is reused every month until changed.'),
        ),
    ]
