# Generated by Django 2.2.7.dev20191011095232 on 2019-10-30 14:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0001_squashed_0093'),
    ]

    operations = [
        migrations.AddField(
            model_name='offeringcomponent',
            name='use_limit_for_billing',
            field=models.BooleanField(default=False, help_text='Charge for usage-based component is based on user-requested limit.'),
        ),
    ]
