# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-30 13:48
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_zabbix', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='trigger',
            name='priority',
            field=models.IntegerField(default=0),
        ),
    ]
