# Generated by Django 2.2.7 on 2019-12-04 09:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0007_offering_enable_dynamic_components'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='offering',
            name='enable_dynamic_components',
        ),
    ]
