# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-25 07:28
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_vmware', '0017_customer_network_pair'),
    ]

    operations = [
        migrations.AddField(
            model_name='port',
            name='runtime_state',
            field=models.CharField(blank=True, max_length=150, verbose_name='runtime state'),
        ),
    ]
