# Generated by Django 2.2.7 on 2019-12-03 11:23

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0001_squashed_0037'),
    ]

    operations = [
        migrations.AlterField(
            model_name='issue',
            name='caller',
            field=models.ForeignKey(blank=True, help_text='Waldur user who has reported the issue.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='created_issues', to=settings.AUTH_USER_MODEL),
        ),
    ]
