# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:02
from django.db import migrations
import waldur_core.core.fields
import waldur_core.cost_tracking.models


class Migration(migrations.Migration):

    dependencies = [
        ('cost_tracking', '0001_squashed_0026'),
    ]

    operations = [
        migrations.AlterField(
            model_name='consumptiondetails',
            name='configuration',
            field=waldur_core.cost_tracking.models.ConsumableItemsField(default=dict, help_text='Current resource configuration.'),
        ),
        migrations.AlterField(
            model_name='consumptiondetails',
            name='consumed_before_update',
            field=waldur_core.cost_tracking.models.ConsumableItemsField(default=dict, help_text='How many consumables were used by resource before last update.'),
        ),
        migrations.AlterField(
            model_name='defaultpricelistitem',
            name='metadata',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, help_text='Details of the item, that corresponds price list item. Example: details of flavor.'),
        ),
        migrations.AlterField(
            model_name='priceestimate',
            name='details',
            field=waldur_core.core.fields.JSONField(default=dict, help_text='Saved scope details. Field is populated on scope deletion.'),
        ),
    ]
