# Generated by Django 2.2.7 on 2020-01-21 10:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0009_offering_parent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='categorycolumn',
            name='widget',
            field=models.CharField(blank=True, help_text='Widget field allows to customise table cell rendering.', max_length=255, null=True),
        ),
    ]
