# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-02-21 13:24
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_jira', '0010_issue_resolution_sla'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='attachment',
            unique_together=set([('issue', 'backend_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='comment',
            unique_together=set([('issue', 'backend_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='issue',
            unique_together=set([('project', 'backend_id')]),
        ),
    ]
