# Generated by Django 2.2.7 on 2019-11-22 17:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0005_offering_plugin_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderitem',
            name='output',
            field=models.TextField(blank=True),
        ),
    ]
