# Generated by Django 2.2.7 on 2020-01-20 12:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0010_allow_null_widget'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='datacite_doi',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]
