# Generated by Django 2.2.9 on 2020-02-21 21:17

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0007_changeemailrequest'),
    ]

    operations = [
        migrations.AddField(
            model_name='changeemailrequest',
            name='uuid',
            field=waldur_core.core.fields.UUIDField(),
        ),
    ]
