# Generated by Django 2.2.7 on 2019-12-11 10:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0001_squashed_0046'),
    ]

    operations = [
        migrations.AddField(
            model_name='volumetype',
            name='is_default',
            field=models.BooleanField(default=False),
        ),
    ]
