# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:02
from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('playbook_jobs', '0005_add_job_deleting_status'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='arguments',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, null=True),
        ),
    ]
