# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-04 20:54
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cost_tracking', '0002_immutable_default_json'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='defaultpricelistitem',
            name='metadata',
        ),
    ]
